{% extends "layout.html" %}
{% block body %}
<div class="deck">

    <p class="developmentversion">
    Celery 3.1 now supports Django out of the box, please see
    <a href="http://docs.celeryproject.org/en/master/django/first-steps-with-django.html"
    >the new tutorial</a></p>
</div>
    {{ body }}
{% endblock %}
